@@ -68,9 +68,6 @@ static gboolean  panel_application_plugin_insert      (PanelApplication       *a
                                                        gint                    unique_id,
                                                        gchar                 **arguments,
                                                        gint                    position);
-static gboolean  panel_application_save_timeout       (gpointer                user_data);
-static void      panel_application_window_destroyed   (GtkWidget              *window,
-                                                       PanelApplication       *application);
 static void      panel_application_dialog_destroyed   (GtkWindow              *dialog,
                                                        PanelApplication       *application);
 static void      panel_application_drag_data_received (PanelWindow            *window,
@@ -97,6 +94,9 @@ static void      panel_application_drag_leave         (GtkWidget              *w
                                                        GdkDragContext         *context,
                                                        guint                   drag_time,
                                                        PanelApplication       *application);
+static void      panel_application_save_window        (PanelApplication       *application,
+                                                       PanelWindow            *window,
+                                                       PanelSaveTypes          save_types);
 
 
 
@@ -127,9 +127,6 @@ struct _PanelApplication
   /* internal list of opened dialogs */
   GSList             *dialogs;
 
-  /* autosave timeout */
-  guint               autosave_timeout_id;
-
 #ifdef GDK_WINDOWING_X11
   guint               wait_for_wm_timeout_id;
 #endif
@@ -198,10 +195,10 @@ static void
 panel_application_init (PanelApplication *application)
 {
   GError *error = NULL;
+  gint    configver;
 
   application->windows = NULL;
   application->dialogs = NULL;
-  application->autosave_timeout_id = 0;
   application->drop_desktop_files = FALSE;
   application->drop_data_ready = FALSE;
   application->drop_occurred = FALSE;
@@ -209,8 +206,9 @@ panel_application_init (PanelApplication *application)
   /* get the xfconf channel (singleton) */
   application->xfconf = panel_properties_get_channel (G_OBJECT (application));
 
-  /* check if we need to launch the migration application */
-  if (!xfconf_channel_has_property (application->xfconf, "/panels"))
+  /* check if we need to migrate configuration */
+  configver = xfconf_channel_get_int (application->xfconf, "/configver", -1);
+  if (G_UNLIKELY (configver < XFCE4_PANEL_CONFIG_VERSION))
     {
       if (!g_spawn_command_line_sync (MIGRATE_BIN, NULL, NULL, NULL, &error))
         {
@@ -225,12 +223,6 @@ panel_application_init (PanelApplication *application)
 
   /* get a factory reference so it never unloads */
   application->factory = panel_module_factory_get ();
-
-  /* start the autosave timeout */
-  application->autosave_timeout_id =
-      g_timeout_add_seconds (AUTOSAVE_INTERVAL,
-                             panel_application_save_timeout,
-                             application);
 }
 
 
@@ -239,26 +231,17 @@ static void
 panel_application_finalize (GObject *object)
 {
   PanelApplication *application = PANEL_APPLICATION (object);
-  GSList           *li;
 
   panel_return_if_fail (application->dialogs == NULL);
 
-  /* stop the autosave timeout */
-  g_source_remove (application->autosave_timeout_id);
-
 #ifdef GDK_WINDOWING_X11
   /* stop autostart timeout */
   if (application->wait_for_wm_timeout_id != 0)
     g_source_remove (application->wait_for_wm_timeout_id);
 #endif
 
-  /* free all windows */
-  for (li = application->windows; li != NULL; li = li->next)
-    {
-      g_signal_handlers_disconnect_by_func (G_OBJECT (li->data),
-          G_CALLBACK (panel_application_window_destroyed), application);
-      gtk_widget_destroy (GTK_WIDGET (li->data));
-    }
+  /* destroy all panels */
+  g_slist_foreach (application->windows, (GFunc) gtk_widget_destroy, NULL);
   g_slist_free (application->windows);
 
   g_object_unref (G_OBJECT (application->factory));
@@ -282,8 +265,9 @@ panel_application_xfconf_window_bindings (PanelApplication *application,
     { "position-locked", G_TYPE_BOOLEAN },
     { "autohide", G_TYPE_BOOLEAN },
     { "span-monitors", G_TYPE_BOOLEAN },
-    { "horizontal", G_TYPE_BOOLEAN },
+    { "mode", G_TYPE_UINT },
     { "size", G_TYPE_UINT },
+    { "nrows", G_TYPE_UINT },
     { "length", G_TYPE_UINT },
     { "length-adjust", G_TYPE_BOOLEAN },
     { "enter-opacity", G_TYPE_UINT },
@@ -299,11 +283,9 @@ panel_application_xfconf_window_bindings (PanelApplication *application,
   };
 
   panel_return_if_fail (XFCONF_IS_CHANNEL (application->xfconf));
-  panel_return_if_fail (g_slist_index (application->windows, window) > -1);
 
   /* create the property base */
-  property_base = g_strdup_printf ("/panels/panel-%d",
-      g_slist_index (application->windows, window));
+  property_base = g_strdup_printf ("/panels/panel-%d", panel_window_get_id (window));
 
   /* bind all the properties */
   panel_properties_bind (application->xfconf, G_OBJECT (window),
@@ -332,74 +314,109 @@ panel_application_load_real (PanelApplication *application)
   gchar        *output_name;
   gint          screen_num;
   GdkDisplay   *display;
+  GValue        val = { 0, };
+  GPtrArray    *panels;
+  gint          panel_id;
 
   panel_return_if_fail (PANEL_IS_APPLICATION (application));
   panel_return_if_fail (XFCONF_IS_CHANNEL (application->xfconf));
 
   display = gdk_display_get_default ();
 
-  /* walk all the panel in the configuration */
-  n_panels = xfconf_channel_get_uint (application->xfconf, "/panels", 0);
-  for (i = 0; i < n_panels; i++)
+  if (xfconf_channel_get_property (application->xfconf, "/panels", &val)
+      && (G_VALUE_HOLDS_UINT (&val)
+          || G_VALUE_HOLDS (&val, PANEL_PROPERTIES_TYPE_VALUE_ARRAY)))
     {
-      screen = NULL;
-
-      /* start the panel directly on the correct screen */
-      g_snprintf (buf, sizeof (buf), "/panels/panel-%u/output-name", i);
-      output_name = xfconf_channel_get_string (application->xfconf, buf, NULL);
-      if (output_name != NULL
-          && strncmp (output_name, "screen-", 7) == 0
-          && sscanf (output_name, "screen-%d", &screen_num) == 1)
+      if (G_VALUE_HOLDS_UINT (&val))
         {
-          if (screen_num < gdk_display_get_n_screens (display))
-            screen = gdk_display_get_screen (display, screen_num);
+          n_panels = g_value_get_uint (&val);
+          panels = NULL;
         }
-      g_free (output_name);
-
-      /* create a new window */
-      window = panel_application_new_window (application, screen, FALSE);
-
-      /* walk all the plugins on the panel */
-      g_snprintf (buf, sizeof (buf), "/panels/panel-%u/plugin-ids", i);
-      array = xfconf_channel_get_arrayv (application->xfconf, buf);
-      if (array == NULL)
-        continue;
-
-      for (j = 0; j < array->len; j++)
+      else
         {
-          /* get the plugin id */
-          value = g_ptr_array_index (array, j);
-          panel_assert (value != NULL);
-          unique_id = g_value_get_int (value);
+          panels = g_value_get_boxed (&val);
+          n_panels = panels->len;
+        }
 
-          /* get the plugin name */
-          g_snprintf (buf, sizeof (buf), "/plugins/plugin-%d", unique_id);
-          name = xfconf_channel_get_string (application->xfconf, buf, NULL);
+      /* walk all the panel in the configuration */
+      for (i = 0; i < n_panels; i++)
+        {
+          screen = NULL;
 
-          /* append the plugin to the panel */
-          if (unique_id < 1 || name == NULL
-              || !panel_application_plugin_insert (application, window,
-                                                   name, unique_id, NULL, -1))
+          /* get the panel id */
+          if (panels != NULL)
             {
-              /* plugin could not be loaded, remove it from the channel */
-              g_snprintf (buf, sizeof (buf), "/panels/plugin-%d", unique_id);
-              if (xfconf_channel_has_property (application->xfconf, buf))
-                xfconf_channel_reset_property (application->xfconf, buf, TRUE);
-
-              /* show warnings */
-              g_message ("Plugin \"%s-%d\" was not found and has been "
-                         "removed from the configuration", name, unique_id);
+              /* get the id from the array */
+              value = g_ptr_array_index (panels, i);
+              panel_assert (value != NULL);
+              panel_id = g_value_get_int (value);
+            }
+          else
+            {
+              /* use the list position if /panels is an uint */
+              panel_id = i;
             }
 
-          g_free (name);
+          /* start the panel directly on the correct screen */
+          g_snprintf (buf, sizeof (buf), "/panels/panel-%d/output-name", panel_id);
+          output_name = xfconf_channel_get_string (application->xfconf, buf, NULL);
+          if (output_name != NULL
+              && strncmp (output_name, "screen-", 7) == 0
+              && sscanf (output_name, "screen-%d", &screen_num) == 1)
+            {
+              if (screen_num < gdk_display_get_n_screens (display))
+                screen = gdk_display_get_screen (display, screen_num);
+            }
+          g_free (output_name);
+
+          /* create a new window */
+          window = panel_application_new_window (application, screen, panel_id, FALSE);
+
+          /* walk all the plugins on the panel */
+          g_snprintf (buf, sizeof (buf), "/panels/panel-%d/plugin-ids", panel_id);
+          array = xfconf_channel_get_arrayv (application->xfconf, buf);
+          if (array == NULL)
+            continue;
+
+          for (j = 0; j < array->len; j++)
+            {
+              /* get the plugin id */
+              value = g_ptr_array_index (array, j);
+              panel_assert (value != NULL);
+              unique_id = g_value_get_int (value);
+
+              /* get the plugin name */
+              g_snprintf (buf, sizeof (buf), "/plugins/plugin-%d", unique_id);
+              name = xfconf_channel_get_string (application->xfconf, buf, NULL);
+
+              /* append the plugin to the panel */
+              if (unique_id < 1 || name == NULL
+                  || !panel_application_plugin_insert (application, window,
+                                                       name, unique_id, NULL, -1))
+                {
+                  /* plugin could not be loaded, remove it from the channel */
+                  g_snprintf (buf, sizeof (buf), "/panels/plugin-%d", unique_id);
+                  if (xfconf_channel_has_property (application->xfconf, buf))
+                    xfconf_channel_reset_property (application->xfconf, buf, TRUE);
+
+                  /* show warnings */
+                  g_message ("Plugin \"%s-%d\" was not found and has been "
+                             "removed from the configuration", name, unique_id);
+                }
+
+              g_free (name);
+            }
+
+          xfconf_array_free (array);
         }
 
-      xfconf_array_free (array);
+      /* free xfconf array or uint */
+      g_value_unset (&val);
     }
 
   /* create empty window if everything else failed */
   if (G_UNLIKELY (application->windows == NULL))
-    panel_application_new_window (application, NULL, TRUE);
+    panel_application_new_window (application, NULL, -1, TRUE);
 }
 
 
@@ -620,13 +637,23 @@ panel_application_plugin_provider_signal (XfcePanelPluginProvider       *provide
                                NULL);
       break;
 
-    case PROVIDER_SIGNAL_WRAP_PLUGIN:
-    case PROVIDER_SIGNAL_UNWRAP_PLUGIN:
+    case PROVIDER_SIGNAL_SHRINK_PLUGIN:
+    case PROVIDER_SIGNAL_UNSHRINK_PLUGIN:
       itembar = gtk_bin_get_child (GTK_BIN (window));
       gtk_container_child_set (GTK_CONTAINER (itembar),
                                GTK_WIDGET (provider),
-                               "wrap",
-                               provider_signal == PROVIDER_SIGNAL_WRAP_PLUGIN,
+                               "shrink",
+                               provider_signal == PROVIDER_SIGNAL_SHRINK_PLUGIN,
+                               NULL);
+      break;
+
+    case PROVIDER_SIGNAL_SMALL_PLUGIN:
+    case PROVIDER_SIGNAL_UNSMALL_PLUGIN:
+      itembar = gtk_bin_get_child (GTK_BIN (window));
+      gtk_container_child_set (GTK_CONTAINER (itembar),
+                               GTK_WIDGET (provider),
+                               "small",
+                               provider_signal == PROVIDER_SIGNAL_SMALL_PLUGIN,
                                NULL);
       break;
 
@@ -758,92 +785,6 @@ panel_application_plugin_insert (PanelApplication  *application,
 
 
 
-static gboolean
-panel_application_save_timeout (gpointer user_data)
-{
-  panel_return_val_if_fail (PANEL_IS_APPLICATION (user_data), FALSE);
-
-  GDK_THREADS_ENTER ();
-
-  panel_application_save (PANEL_APPLICATION (user_data), TRUE);
-
-  GDK_THREADS_LEAVE ();
-
-  return TRUE;
-}
-
-
-
-static void
-panel_application_window_destroyed (GtkWidget        *window,
-                                    PanelApplication *application)
-{
-  guint      n;
-  gchar     *property;
-  GSList    *li, *lnext;
-  gboolean   passed_destroyed_window = FALSE;
-  GtkWidget *itembar;
-
-  panel_return_if_fail (PANEL_IS_WINDOW (window));
-  panel_return_if_fail (PANEL_IS_APPLICATION (application));
-  panel_return_if_fail (g_slist_find (application->windows, window) != NULL);
-
-  /* leave if the application is locked */
-  if (panel_application_get_locked (application))
-    return;
-
-  /* we need to update the bindings of all the panels... */
-  for (li = application->windows, n = 0; li != NULL; li = lnext, n++)
-    {
-      lnext = li->next;
-
-      /* TODO, this might go wrong when only 1 window is locked */
-      if (panel_window_get_locked (li->data))
-        continue;
-
-      if (passed_destroyed_window)
-        {
-          /* save this panel again at it's new position */
-          panel_properties_unbind (G_OBJECT (li->data));
-          panel_application_xfconf_window_bindings (application,
-                                                    PANEL_WINDOW (li->data),
-                                                    TRUE);
-        }
-      else if (li->data == window)
-        {
-          /* disconnect bindings from this panel */
-          panel_properties_unbind (G_OBJECT (window));
-
-          /* remove all the plugins from the itembar */
-          itembar = gtk_bin_get_child (GTK_BIN (window));
-          gtk_container_foreach (GTK_CONTAINER (itembar),
-              panel_application_plugin_remove, NULL);
-
-          /* remove from the internal list */
-          application->windows = g_slist_delete_link (application->windows, li);
-
-          /* keep updating the bindings for the remaining windows */
-          passed_destroyed_window = TRUE;
-        }
-    }
-
-  /* remove the last property from the channel */
-  property = g_strdup_printf ("/panels/panel-%u", n - 1);
-  panel_assert (n - 1 == g_slist_length (application->windows));
-  xfconf_channel_reset_property (application->xfconf, property, TRUE);
-  g_free (property);
-
-  /* schedule a save to store the new number of panels */
-  panel_application_save (application, FALSE);
-
-  /* quit if there are no windows */
-  /* TODO, allow removing all windows and ask user what to do */
-  if (application->windows == NULL)
-    gtk_main_quit ();
-}
-
-
-
 static void
 panel_application_windows_block_autohide (PanelApplication *application,
                                           gboolean          blocked)
@@ -894,6 +835,7 @@ panel_application_drag_data_received (PanelWindow      *window,
   PanelApplication  *application;
   GtkWidget         *provider;
   gboolean           succeed = FALSE;
+  gboolean           save_application = FALSE;
   const gchar       *name;
   guint              old_position;
   gchar            **uris;
@@ -984,6 +926,9 @@ panel_application_drag_data_received (PanelWindow      *window,
 
               /* send all the needed panel information to the plugin */
               panel_window_set_povider_info (window, provider, TRUE);
+
+              /* moved between panels, save everything */
+              save_application = TRUE;
             }
 
           /* everything went fine */
@@ -1013,9 +958,11 @@ panel_application_drag_data_received (PanelWindow      *window,
           break;
         }
 
-      /* save the panel configuration if we succeeded */
-      if (G_LIKELY (succeed))
-        panel_application_save (application, FALSE);
+      /* save the new plugin ids */
+      if (G_LIKELY (save_application))
+        panel_application_save (application, SAVE_PLUGIN_IDS);
+      else if (succeed)
+        panel_application_save_window (application, window, SAVE_PLUGIN_IDS);
 
       /* tell the peer that we handled the drop */
       gtk_drag_finish (context, succeed, FALSE, drag_time);
@@ -1156,6 +1103,104 @@ panel_application_drag_leave (GtkWidget        *window,
 
 
 
+static gboolean
+panel_application_window_id_exists (PanelApplication *application,
+                                    gint              id)
+{
+  GSList *li;
+
+  for (li = application->windows; li != NULL; li = li->next)
+    if (panel_window_get_id (li->data) == id)
+      return TRUE;
+
+  return FALSE;
+}
+
+
+
+static void
+panel_application_save_window (PanelApplication *application,
+                               PanelWindow      *window,
+                               PanelSaveTypes    save_types)
+{
+  GList                   *children, *lp;
+  GtkWidget               *itembar;
+  XfcePanelPluginProvider *provider;
+  gchar                    buf[50];
+  XfconfChannel           *channel = application->xfconf;
+  GPtrArray               *array = NULL;
+  GValue                  *value;
+  gint                     plugin_id;
+  gint                     panel_id;
+
+  /* skip this window if it is locked */
+  if (panel_window_get_locked (window)
+      || !PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_IDS | SAVE_PLUGIN_PROVIDERS))
+    return;
+
+  panel_id = panel_window_get_id (window);
+  panel_debug (PANEL_DEBUG_APPLICATION,
+               "saving /panels/panel-%d: ids=%s, providers=%s",
+               panel_id,
+               PANEL_DEBUG_BOOL (PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_IDS)),
+               PANEL_DEBUG_BOOL (PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_PROVIDERS)));
+
+  /* get the itembar children */
+  itembar = gtk_bin_get_child (GTK_BIN (window));
+  children = gtk_container_get_children (GTK_CONTAINER (itembar));
+
+  /* only cleanup and continue if there are no children */
+  if (PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_IDS))
+    {
+      if (G_UNLIKELY (children == NULL))
+        {
+          g_snprintf (buf, sizeof (buf), "/panels/panel-%d/plugin-ids", panel_id);
+          if (xfconf_channel_has_property (channel, buf))
+            xfconf_channel_reset_property (channel, buf, FALSE);
+          return;
+        }
+
+      array = g_ptr_array_new ();
+    }
+
+  /* walk all the plugin children */
+  for (lp = children; lp != NULL; lp = lp->next)
+    {
+      provider = XFCE_PANEL_PLUGIN_PROVIDER (lp->data);
+
+      if (array != NULL)
+        {
+          plugin_id = xfce_panel_plugin_provider_get_unique_id (provider);
+
+          /* add plugin id to the array */
+          value = g_new0 (GValue, 1);
+          g_value_init (value, G_TYPE_INT);
+          g_value_set_int (value, plugin_id);
+          g_ptr_array_add (array, value);
+
+          /* make sure the plugin type-name is store in the plugin item */
+          g_snprintf (buf, sizeof (buf), "/plugins/plugin-%d", plugin_id);
+          xfconf_channel_set_string (channel, buf, xfce_panel_plugin_provider_get_name (provider));
+        }
+
+      /* ask the plugin to save */
+      if (PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_PROVIDERS))
+        xfce_panel_plugin_provider_save (provider);
+    }
+
+  if (array != NULL)
+    {
+      /* store the plugin ids for this panel */
+      g_snprintf (buf, sizeof (buf), "/panels/panel-%d/plugin-ids", panel_id);
+      xfconf_channel_set_arrayv (channel, buf, array);
+      xfconf_array_free (array);
+    }
+
+  g_list_free (children);
+}
+
+
+
 PanelApplication *
 panel_application_get (void)
 {
@@ -1227,79 +1272,47 @@ panel_application_load (PanelApplication  *application,
 
 void
 panel_application_save (PanelApplication *application,
-                        gboolean          save_plugin_providers)
+                        PanelSaveTypes    save_types)
 {
-  GSList                  *li;
-  GList                   *children, *lp;
-  GtkWidget               *itembar;
-  XfcePanelPluginProvider *provider;
-  guint                    i;
-  gchar                    buf[50];
-  XfconfChannel           *channel = application->xfconf;
-  GPtrArray               *array;
-  GValue                  *value;
+  GSList        *li;
+  XfconfChannel *channel = application->xfconf;
+  GValue        *value;
+  GPtrArray     *panels = NULL;
+  gint           panel_id;
 
   panel_return_if_fail (PANEL_IS_APPLICATION (application));
   panel_return_if_fail (XFCONF_IS_CHANNEL (channel));
 
-  for (li = application->windows, i = 0; li != NULL; li = li->next, i++)
+  /* leave if the whole application is locked */
+  if (xfconf_channel_is_property_locked (channel, "/panels"))
+    return;
+
+  if (PANEL_HAS_FLAG (save_types, SAVE_PANEL_IDS))
+    panels = g_ptr_array_new ();
+
+  for (li = application->windows; li != NULL; li = li->next)
     {
-      /* skip this window if it is locked */
-      if (panel_window_get_locked (li->data))
-        continue;
-
-      panel_debug (PANEL_DEBUG_APPLICATION,
-                   "saving /panels/panel-%u, save-plugins=%s",
-                   i, PANEL_DEBUG_BOOL (save_plugin_providers));
-
-      /* get the itembar children */
-      itembar = gtk_bin_get_child (GTK_BIN (li->data));
-      children = gtk_container_get_children (GTK_CONTAINER (itembar));
-
-      /* only cleanup and continue if there are no children */
-      if (G_UNLIKELY (children == NULL))
+      if (panels != NULL)
         {
-          g_snprintf (buf, sizeof (buf), "/panels/panel-%u/plugin-ids", i);
-          if (xfconf_channel_has_property (channel, buf))
-            xfconf_channel_reset_property (channel, buf, FALSE);
-          continue;
-        }
-
-      /* create empty pointer array */
-      array = g_ptr_array_new ();
-
-      /* walk all the plugin children */
-      for (lp = children; lp != NULL; lp = lp->next)
-        {
-          provider = XFCE_PANEL_PLUGIN_PROVIDER (lp->data);
-
-          /* add plugin id to the array */
+          /* store the panel id */
           value = g_new0 (GValue, 1);
+          panel_id = panel_window_get_id (li->data);
           g_value_init (value, G_TYPE_INT);
-          g_value_set_int (value,
-              xfce_panel_plugin_provider_get_unique_id (provider));
-          g_ptr_array_add (array, value);
-
-          /* make sure the plugin type is store in the plugin item */
-          g_snprintf (buf, sizeof (buf), "/plugins/plugin-%d", g_value_get_int (value));
-          xfconf_channel_set_string (channel, buf, xfce_panel_plugin_provider_get_name (provider));
-
-          /* ask the plugin to save */
-          if (save_plugin_providers)
-            xfce_panel_plugin_provider_save (provider);
+          g_value_set_int (value, panel_id);
+          g_ptr_array_add (panels, value);
         }
 
-      /* store the plugins for this panel */
-      g_snprintf (buf, sizeof (buf), "/panels/panel-%u/plugin-ids", i);
-      xfconf_channel_set_arrayv (channel, buf, array);
-
-      g_list_free (children);
-      xfconf_array_free (array);
+      /* save the panel settings */
+      panel_application_save_window (application, li->data, save_types);
     }
 
-  /* store the number of panels */
-  if (!xfconf_channel_is_property_locked (channel, "/panels"))
-    xfconf_channel_set_uint (channel, "/panels", i);
+  if (panels != NULL)
+    {
+      /* store the panel ids */
+      if (!xfconf_channel_set_arrayv (channel, "/panels", panels))
+        g_warning ("Failed to store the number of panels");
+      xfconf_array_free (panels);
+    }
 }
 
 
@@ -1347,14 +1360,12 @@ panel_application_add_new_item (PanelApplication  *application,
                                 const gchar       *plugin_name,
                                 gchar            **arguments)
 {
-  gint         nth = 0;
-  GSList      *li;
-  gboolean     active;
-  PanelWindow *window;
+  PanelWindow *window = NULL;
+  gint         panel_id;
 
   panel_return_if_fail (PANEL_IS_APPLICATION (application));
   panel_return_if_fail (plugin_name != NULL);
-  panel_return_if_fail (g_slist_length (application->windows) > 0);
+  panel_return_if_fail (application->windows != NULL);
 
   /* leave if the config is locked */
   if (panel_application_get_locked (application))
@@ -1362,30 +1373,38 @@ panel_application_add_new_item (PanelApplication  *application,
 
   if (panel_module_factory_has_module (application->factory, plugin_name))
     {
-      /* find a suitable window if there are 2 or more windows */
+      /* find a suitable panel if there are 2 or more panel */
       if (LIST_HAS_TWO_OR_MORE_ENTRIES (application->windows))
         {
-          /* try to find an avtive panel */
-          for (li = application->windows, nth = 0; li != NULL; li = li->next, nth++)
+          /* ask the user to select a panel */
+          panel_id = panel_dialogs_choose_panel (application);
+          if (panel_id == -1)
             {
-              g_object_get (G_OBJECT (li->data), "active", &active, NULL);
-              if (active)
-                break;
+              /* cancel was clicked */
+              return;
             }
-
-          /* no active panel found, ask user to select a panel, leave when
-           * the cancel button is pressed */
-          if (li == NULL
-              && (nth = panel_dialogs_choose_panel (application)) == -1)
-            return;
+          else
+            {
+              /* get panel from the id */
+              window = panel_application_get_window (application, panel_id);
+            }
+        }
+      else
+        {
+          /* get the first (and only) window */
+          window = g_slist_nth_data (application->windows, 0);
         }
 
-      /* add the plugin to the end of the choosen window */
-      window = g_slist_nth_data (application->windows, nth);
-      if (!panel_window_get_locked (window))
+      if (window != NULL && !panel_window_get_locked (window))
         {
-          panel_application_plugin_insert (application, window,
-                                           plugin_name, -1, arguments, -1);
+          /* insert plugin at the end of the panel */
+          if (panel_application_plugin_insert (application, window,
+                                               plugin_name, -1,
+                                               arguments, -1))
+            {
+              /* save the new plugin ids */
+              panel_application_save_window (application, window, SAVE_PLUGIN_IDS);
+            }
         }
     }
   else
@@ -1400,44 +1419,49 @@ panel_application_add_new_item (PanelApplication  *application,
 PanelWindow *
 panel_application_new_window (PanelApplication *application,
                               GdkScreen        *screen,
+                              gint              panel_id,
                               gboolean          new_window)
 {
-  GtkWidget *window;
-  GtkWidget *itembar;
-  gchar     *property;
-  gint       idx;
+  GtkWidget          *window;
+  GtkWidget          *itembar;
+  gchar              *property;
+  gint                idx;
+  static const gchar *props[] = { "mode", "size", "nrows" };
+  guint               i;
+  gchar              *position;
+  static gint         unqiue_id_counter = 1;
 
   panel_return_val_if_fail (PANEL_IS_APPLICATION (application), NULL);
   panel_return_val_if_fail (screen == NULL || GDK_IS_SCREEN (screen), NULL);
   panel_return_val_if_fail (XFCONF_IS_CHANNEL (application->xfconf), NULL);
+  panel_return_val_if_fail (new_window || !panel_application_window_id_exists (application, panel_id), NULL);
+
+  if (new_window)
+    {
+      /* get a new unique id */
+      panel_id = unqiue_id_counter;
+      while (panel_application_window_id_exists (application, panel_id))
+        panel_id = ++unqiue_id_counter;
+    }
 
   /* create panel window */
-  window = panel_window_new (screen);
-
-  /* monitor window destruction */
-  g_signal_connect (G_OBJECT (window), "destroy",
-      G_CALLBACK (panel_application_window_destroyed), application);
+  window = panel_window_new (screen, panel_id);
 
   /* add the window to internal list */
   application->windows = g_slist_append (application->windows, window);
 
-  /* flush the window properties */
   if (new_window)
     {
-      /* remove the xfconf properties */
-      idx = g_slist_index (application->windows, window);
-      property = g_strdup_printf ("/panels/panel-%d", idx);
+      /* remove the old xfconf properties to be sure */
+      property = g_strdup_printf ("/panels/panel-%d", panel_id);
       xfconf_channel_reset_property (application->xfconf, property, TRUE);
       g_free (property);
-
-      /* set default position */
-      g_object_set (G_OBJECT (window), "position", "p=0;x=100;y=100", NULL);
     }
 
   /* add the itembar */
   itembar = panel_itembar_new ();
-  exo_binding_new (G_OBJECT (window), "horizontal", G_OBJECT (itembar), "horizontal");
-  exo_binding_new (G_OBJECT (window), "size", G_OBJECT (itembar), "size");
+  for (i = 0; i < G_N_ELEMENTS (props); i++)
+    exo_binding_new (G_OBJECT (window), props[i], G_OBJECT (itembar), props[i]);
   gtk_container_add (GTK_CONTAINER (window), itembar);
   gtk_widget_show (itembar);
 
@@ -1459,45 +1483,102 @@ panel_application_new_window (PanelApplication *application,
   /* add the xfconf bindings */
   panel_application_xfconf_window_bindings (application, PANEL_WINDOW (window), FALSE);
 
-  /* make sure the position of the panel is always saved else
-   * the new window won't be visible on restart */
+  /* make sure the panel has a valid position, else it is not visible */
+  if (!panel_window_has_position (PANEL_WINDOW (window)))
+    {
+      if (!new_window)
+        g_message ("No panel position set, restoring default");
+
+      /* create a position so not all panels overlap */
+      idx = g_slist_index (application->windows, window);
+      position = g_strdup_printf ("p=0;x=100;y=%d", 30 + (idx * (48 + 10)));
+      g_object_set (G_OBJECT (window), "position", position, NULL);
+      g_free (position);
+    }
+
+  /* save the new panel layout */
   if (new_window)
-    g_object_notify (G_OBJECT (window), "position");
+    panel_application_save (application, SAVE_PANEL_IDS);
 
   return PANEL_WINDOW (window);
 }
 
 
 
-guint
-panel_application_get_n_windows (PanelApplication *application)
+void
+panel_application_remove_window (PanelApplication *application,
+                                 PanelWindow      *window)
 {
-  panel_return_val_if_fail (PANEL_IS_APPLICATION (application), 0);
+  gchar     *property;
+  GtkWidget *itembar;
+  gint       panel_id;
 
-  return g_slist_length (application->windows);
+  panel_return_if_fail (PANEL_IS_WINDOW (window));
+  panel_return_if_fail (PANEL_IS_APPLICATION (application));
+  panel_return_if_fail (g_slist_find (application->windows, window) != NULL);
+
+  /* leave if the application or window is locked */
+  if (panel_application_get_locked (application)
+      || panel_window_get_locked (PANEL_WINDOW (window)))
+    return;
+
+  panel_id = panel_window_get_id (PANEL_WINDOW (window));
+  panel_debug (PANEL_DEBUG_APPLICATION,
+               "removing configuration and plugins of panel %d",
+               panel_id);
+
+  /* remove from the internal list */
+  application->windows = g_slist_remove (application->windows, window);
+
+  /* disconnect bindings from this panel */
+  panel_properties_unbind (G_OBJECT (window));
+
+  /* set all the plugins on this panel the remove signal */
+  itembar = gtk_bin_get_child (GTK_BIN (window));
+  gtk_container_foreach (GTK_CONTAINER (itembar),
+      panel_application_plugin_remove, NULL);
+
+  /* destroy */
+  gtk_widget_destroy (GTK_WIDGET (window));
+
+  /* remove the panel settings */
+  property = g_strdup_printf ("/panels/panel-%d", panel_id);
+  xfconf_channel_reset_property (application->xfconf, property, TRUE);
+  g_free (property);
+
+  /* save changed panel ids */
+  panel_application_save (application, SAVE_PANEL_IDS);
+
+  /* quit if there are no windows */
+  /* TODO, allow removing all windows and ask user what to do */
+  if (application->windows == NULL)
+    gtk_main_quit ();
 }
 
 
 
-gint
-panel_application_get_window_index (PanelApplication *application,
-                                    PanelWindow      *window)
+GSList *
+panel_application_get_windows (PanelApplication *application)
 {
-  panel_return_val_if_fail (PANEL_IS_APPLICATION (application), 0);
-  panel_return_val_if_fail (PANEL_IS_WINDOW (window), 0);
-
-  return g_slist_index (application->windows, window);
+  panel_return_val_if_fail (PANEL_IS_APPLICATION (application), NULL);
+  return application->windows;
 }
 
 
 
 PanelWindow *
-panel_application_get_nth_window (PanelApplication *application,
-                                  guint             idx)
+panel_application_get_window (PanelApplication  *application,
+                              gint               panel_id)
 {
-  panel_return_val_if_fail (PANEL_IS_APPLICATION (application), 0);
+  GSList *li;
 
-  return g_slist_nth_data (application->windows, idx);
+  panel_return_val_if_fail (PANEL_IS_APPLICATION (application), NULL);
+
+  for (li = application->windows; li != NULL; li = li->next)
+    if (panel_window_get_id (li->data) == panel_id)
+      return li->data;
+
+  return NULL;
 }
 
 
