@@ -191,6 +191,7 @@ panel_item_dialog_init (PanelItemDialog *dialog)
   gtk_window_set_icon_name (GTK_WINDOW (dialog), GTK_STOCK_ADD);
   gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
   gtk_window_set_default_size (GTK_WINDOW (dialog), 350, 450);
+  gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_NORMAL);
 
   dialog->add_button = gtk_button_new_from_stock (GTK_STOCK_ADD);
   gtk_widget_show (dialog->add_button);
@@ -813,6 +814,20 @@ panel_item_dialog_show (PanelWindow *window)
 
 
 
+void
+panel_item_dialog_show_from_id (gint panel_id)
+{
+  PanelApplication *application;
+  PanelWindow      *window;
+
+  application = panel_application_get ();
+  window = panel_application_get_window (application, panel_id);
+  panel_item_dialog_show (window);
+  g_object_unref (G_OBJECT (application));
+}
+
+
+
 gboolean
 panel_item_dialog_visible (void)
 {
