@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2008-2010 Nick Schermer <nick@xfce.org>
+ * Copyright (C) 2008-2011 Nick Schermer <nick@xfce.org>
+ * Copyright (C)      2011 Andrzej <ndrwrdck@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +29,9 @@
 
 #include <panel/panel-itembar.h>
 
+#define IS_HORIZONTAL(itembar) ((itembar)->mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL)
+#define HIGHLIGHT_SIZE         2
+
 
 
 typedef struct _PanelItembarChild PanelItembarChild;
@@ -82,36 +86,47 @@ struct _PanelItembar
 {
   GtkContainer __parent__;
 
-  GSList         *children;
+  GSList              *children;
 
   /* some properties we clone from the panel window */
-  guint           horizontal : 1;
-  guint           size;
+  XfcePanelPluginMode  mode;
+  gint                 size;
+  gint                 nrows;
 
   /* dnd support */
-  gint            highlight_index;
-  gint            highlight_x, highlight_y;
+  gint                 highlight_index;
+  gint                 highlight_x, highlight_y;
 };
 
+typedef enum
+{
+  CHILD_OPTION_NONE,
+  CHILD_OPTION_EXPAND,
+  CHILD_OPTION_SHRINK,
+  CHILD_OPTION_SMALL
+}
+ChildOptions;
+
 struct _PanelItembarChild
 {
-  GtkWidget *widget;
-  guint      expand : 1;
-  guint      wrap : 1;
+  GtkWidget    *widget;
+  ChildOptions  option;
 };
 
 enum
 {
   PROP_0,
-  PROP_HORIZONTAL,
-  PROP_SIZE
+  PROP_MODE,
+  PROP_SIZE,
+  PROP_NROWS
 };
 
 enum
 {
   CHILD_PROP_0,
   CHILD_PROP_EXPAND,
-  CHILD_PROP_WRAP
+  CHILD_PROP_SHRINK,
+  CHILD_PROP_SMALL
 };
 
 enum
@@ -164,17 +179,25 @@ panel_itembar_class_init (PanelItembarClass *klass)
                   G_TYPE_NONE, 0);
 
   g_object_class_install_property (gobject_class,
-                                   PROP_HORIZONTAL,
-                                   g_param_spec_boolean ("horizontal",
-                                                         NULL, NULL,
-                                                         TRUE,
-                                                         EXO_PARAM_WRITABLE));
+                                   PROP_MODE,
+                                   g_param_spec_enum ("mode",
+                                                      NULL, NULL,
+                                                      XFCE_TYPE_PANEL_PLUGIN_MODE,
+                                                      XFCE_PANEL_PLUGIN_MODE_HORIZONTAL,
+                                                      EXO_PARAM_WRITABLE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_SIZE,
                                    g_param_spec_uint ("size",
                                                       NULL, NULL,
-                                                      16, 128, 48,
+                                                      16, 128, 30,
+                                                      EXO_PARAM_WRITABLE));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_NROWS,
+                                   g_param_spec_uint ("nrows",
+                                                      NULL, NULL,
+                                                      1, 6, 1,
                                                       EXO_PARAM_WRITABLE));
 
   gtk_container_class_install_child_property (gtkcontainer_class,
@@ -185,8 +208,15 @@ panel_itembar_class_init (PanelItembarClass *klass)
                                                                     EXO_PARAM_READWRITE));
 
   gtk_container_class_install_child_property (gtkcontainer_class,
-                                              CHILD_PROP_WRAP,
-                                              g_param_spec_boolean ("wrap",
+                                              CHILD_PROP_SHRINK,
+                                              g_param_spec_boolean ("shrink",
+                                                                    NULL, NULL,
+                                                                    FALSE,
+                                                                    EXO_PARAM_READWRITE));
+
+  gtk_container_class_install_child_property (gtkcontainer_class,
+                                              CHILD_PROP_SMALL,
+                                              g_param_spec_boolean ("small",
                                                                     NULL, NULL,
                                                                     FALSE,
                                                                     EXO_PARAM_READWRITE));
@@ -198,8 +228,9 @@ static void
 panel_itembar_init (PanelItembar *itembar)
 {
   itembar->children = NULL;
-  itembar->horizontal = TRUE;
+  itembar->mode = XFCE_PANEL_PLUGIN_MODE_HORIZONTAL;
   itembar->size = 30;
+  itembar->nrows = 1;
   itembar->highlight_index = -1;
 
   GTK_WIDGET_SET_FLAGS (GTK_WIDGET (itembar), GTK_NO_WINDOW);
@@ -219,14 +250,18 @@ panel_itembar_set_property (GObject      *object,
 
   switch (prop_id)
     {
-    case PROP_HORIZONTAL:
-      itembar->horizontal = g_value_get_boolean (value);
+    case PROP_MODE:
+      itembar->mode = g_value_get_enum (value);
       break;
 
     case PROP_SIZE:
       itembar->size = g_value_get_uint (value);
       break;
 
+    case PROP_NROWS:
+      itembar->nrows = g_value_get_uint (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -261,7 +296,12 @@ panel_itembar_finalize (GObject *object)
   (*G_OBJECT_CLASS (panel_itembar_parent_class)->finalize) (object);
 }
 
+#define CHILD_FITS_IN_ROW(child_req, itembar) \
+  ((IS_HORIZONTAL (itembar) && child_req.height <= (itembar)->size) \
+   || (!IS_HORIZONTAL (itembar) && child_req.width <= (itembar)->size))
 
+#define CHILD_LENGTH(child_req, itembar) \
+  (IS_HORIZONTAL (itembar) ? child_req.width : child_req.height)
 
 static void
 panel_itembar_size_request (GtkWidget      *widget,
@@ -270,76 +310,86 @@ panel_itembar_size_request (GtkWidget      *widget,
   PanelItembar      *itembar = PANEL_ITEMBAR (widget);
   GSList            *li;
   PanelItembarChild *child;
-  GtkRequisition     child_requisition;
+  GtkRequisition     child_req;
   gint               border_width;
-  gint               row_length = 0;
+  gint               row_max_size;
+  gint               col_count;
+  gint               rows_size;
+  gint               total_len;
+  gint               child_len;
 
-  /* intialize the requisition, we always set the panel height */
-  if (itembar->horizontal)
-    {
-      requisition->height = itembar->size;
-      requisition->width = 0;
-    }
-  else
-    {
-      requisition->height = 0;
-      requisition->width = itembar->size;
-    }
+  /* total length we request */
+  total_len = 0;
 
-  /* get the size requests of the children and use the longest row for
-   * the requested width/height when we have wrap items */
-  for (li = itembar->children; li != NULL; li = g_slist_next (li))
+  /* counter for small child packing */
+  row_max_size = 0;
+  col_count = 0;
+
+  for (li = itembar->children; li != NULL; li = li->next)
     {
       child = li->data;
+
       if (G_LIKELY (child != NULL))
         {
           if (!GTK_WIDGET_VISIBLE (child->widget))
             continue;
 
-          gtk_widget_size_request (child->widget, &child_requisition);
+          /* get the child's size request */
+          gtk_widget_size_request (child->widget, &child_req);
 
-          if (G_LIKELY (!child->wrap))
+          /* check if the small child fits in a row */
+          if (child->option == CHILD_OPTION_SMALL
+              && itembar->nrows > 1
+              && CHILD_FITS_IN_ROW (child_req, itembar))
             {
-              if (itembar->horizontal)
-                row_length += child_requisition.width;
-              else
-                row_length += child_requisition.height;
+              child_len = CHILD_LENGTH (child_req, itembar);
+
+              /* make sure we have enough space for all the children on the row.
+               * so add the difference between the largest child in this column */
+              if (child_len > row_max_size)
+                {
+                  total_len += child_len - row_max_size;
+                  row_max_size = child_len;
+                }
+
+              /* reset to new row if all columns are filled */
+              if (++col_count >= itembar->nrows)
+                {
+                  col_count = 0;
+                  row_max_size = 0;
+                }
             }
-          else
+          else /* expanding or normal item */
             {
-              /* add to size for new wrap element */
-              if (itembar->horizontal)
-                {
-                  requisition->height += itembar->size;
-                  requisition->width = MAX (requisition->width, row_length);
-                }
-              else
-                {
-                  requisition->width += itembar->size;
-                  requisition->height = MAX (requisition->height, row_length);
-                }
+              total_len += CHILD_LENGTH (child_req, itembar);
 
-              /* reset length for new row */
-              row_length = 0;
+              /* reset column packing */
+              col_count = 0;
+              row_max_size = 0;
             }
         }
       else
         {
           /* this noop item is the dnd position */
-          row_length += itembar->size;
+          total_len += HIGHLIGHT_SIZE;
         }
     }
 
-  /* also take the last row_length into account */
-  if (itembar->horizontal)
-    requisition->width = MAX (requisition->width, row_length);
-  else
-    requisition->height = MAX (requisition->height, row_length);
+  /* the size property stored in the itembar is that of a single row */
+  rows_size = itembar->size * itembar->nrows;
 
-  /* add border width */
+  /* return the total size */
   border_width = GTK_CONTAINER (widget)->border_width * 2;
-  requisition->height += border_width;
-  requisition->width += border_width;
+  if (IS_HORIZONTAL (itembar))
+    {
+      requisition->width = total_len + border_width;
+      requisition->height = rows_size + border_width;
+    }
+  else
+    {
+      requisition->height = total_len + border_width;
+      requisition->width = rows_size + border_width;
+    }
 }
 
 
@@ -349,181 +399,269 @@ panel_itembar_size_allocate (GtkWidget     *widget,
                              GtkAllocation *allocation)
 {
   PanelItembar      *itembar = PANEL_ITEMBAR (widget);
-  GSList            *li, *lp;
+  GSList            *lp;
   PanelItembarChild *child;
   GtkRequisition     child_req;
   GtkAllocation      child_alloc;
-  guint              n_expand_children;
-  guint              row;
-  gint               expand_length, border_width;
-  gint               expand_length_avail;
-  gint               expand_length_req;
-  gint               length_req, length;
-  gint               alloc_length;
+  gint               border_width;
+  gint               expand_len_avail, expand_len_req;
+  gint               shrink_len_avail, shrink_len_req;
+  gint               itembar_len;
   gint               x, y;
+  gint               x_init, y_init;
   gboolean           expand_children_fit;
+  gint               new_len;
+  gint               child_len;
+  gint               row_max_size;
+  gint               col_count;
+  gint               rows_size;
 
+  /* the maximum allocation is limited by that of the
+   * panel window, so take over the assigned allocation */
   widget->allocation = *allocation;
 
   border_width = GTK_CONTAINER (widget)->border_width;
 
-  if (itembar->horizontal)
-    expand_length = allocation->width - 2 * border_width;
+  if (IS_HORIZONTAL (itembar))
+    itembar_len = allocation->width - 2 * border_width;
   else
-    expand_length = allocation->height - 2 * border_width;
+    itembar_len = allocation->height - 2 * border_width;
 
-  /* loop for wrap items */
-  for (row = 0, li = itembar->children; li != NULL; li = g_slist_next (li), row++)
+  /* init the remaining space for expanding plugins */
+  expand_len_avail = itembar_len;
+  expand_len_req = 0;
+
+  /* init the total size of shrinking plugins */
+  shrink_len_avail = 0;
+  shrink_len_req = 0;
+
+  /* init counters for small child packing */
+  row_max_size = 0;
+  col_count = 0;
+
+  /* get information about the expandable lengths */
+  for (lp = itembar->children; lp != NULL; lp = lp->next)
     {
-      expand_length_avail = expand_length;
-      expand_length_req = 0;
-      n_expand_children = 0;
-
-      /* get information about the expandable lengths */
-      for (lp = li; lp != NULL; lp = g_slist_next (lp))
+      child = lp->data;
+      if (G_LIKELY (child != NULL))
         {
-          child = lp->data;
-          if (G_LIKELY (child != NULL))
-            {
-              if (!GTK_WIDGET_VISIBLE (child->widget))
-                continue;
-
-              /* continue allocating until we hit a wrap child */
-              if (G_UNLIKELY (child->wrap))
-                break;
-
-              gtk_widget_get_child_requisition (child->widget, &child_req);
-              length = itembar->horizontal ? child_req.width : child_req.height;
-
-              if (G_UNLIKELY (child->expand))
-                {
-                  n_expand_children++;
-                  expand_length_req += length;
-                }
-              else
-                {
-                  expand_length_avail -= length;
-                }
-            }
-          else
-            {
-              expand_length_avail -= itembar->size;
-            }
-        }
-
-      /* set start coordinates for the items in the row*/
-      x = allocation->x + border_width;
-      y = allocation->y + border_width;
-      if (itembar->horizontal)
-        y += row * itembar->size;
-      else
-        x += row * itembar->size;
-
-      /* whether the expandable items fit on this row; we use this
-       * as a fast-path when there are expanding items on a panel with
-       * not really enough length to expand (ie. items make the panel grow,
-       * not the length set by the user) */
-      expand_children_fit = expand_length_req == expand_length_avail;
-      if (expand_length_avail < 0)
-        expand_length_avail = 0;
-
-      /* allocate the children on this row */
-      for (; li != NULL; li = g_slist_next (li))
-        {
-          child = li->data;
-
-          /* the highlight item for which we keep some spare space */
-          if (G_UNLIKELY (child == NULL))
-            {
-              itembar->highlight_x = x;
-              itembar->highlight_y = y;
-              expand_length_avail -= itembar->size;
-
-              if (itembar->horizontal)
-                x += itembar->size;
-              else
-                y += itembar->size;
-
-              continue;
-            }
-
           if (!GTK_WIDGET_VISIBLE (child->widget))
             continue;
 
           gtk_widget_get_child_requisition (child->widget, &child_req);
 
-          child_alloc.x = x;
-          child_alloc.y = y;
+          child_len = CHILD_LENGTH (child_req, itembar);
 
-          if (child->wrap)
+          if (G_UNLIKELY (child->option == CHILD_OPTION_SMALL
+              && itembar->nrows > 1
+              && CHILD_FITS_IN_ROW (child_req, itembar)))
             {
-              /* if there is any expanding length available use it for the
-               * wrapping plugin to improve accessibility */
-              if (expand_length_avail > 0)
+              /* extract from the available space */
+              if (child_len > row_max_size)
                 {
-                  if (itembar->horizontal)
-                    {
-                      child_alloc.height = itembar->size;
-                      child_alloc.width = expand_length_avail;
-                    }
-                  else
-                    {
-                      child_alloc.width = itembar->size;
-                      child_alloc.height = expand_length_avail;
-                    }
-                }
-              else
-                {
-                  /* hide it */
-                  child_alloc.width = child_alloc.height = 0;
+                  expand_len_avail -= child_len - row_max_size;
+                  row_max_size = child_len;
                 }
 
-              gtk_widget_size_allocate (child->widget, &child_alloc);
-
-              /* stop and continue to the next row */
-              break;
-            }
-
-          if (G_UNLIKELY (!expand_children_fit && child->expand))
-            {
-              /* try to equally distribute the length between the expanding plugins */
-              length_req = itembar->horizontal ? child_req.width : child_req.height;
-              panel_assert (n_expand_children > 0);
-              if (G_LIKELY (expand_length_req > 0 || length_req > 0))
-                alloc_length = expand_length_avail * length_req / expand_length_req;
-              else
-                alloc_length = expand_length_avail / n_expand_children;
-
-              if (itembar->horizontal)
-                child_alloc.width = alloc_length;
-              else
-                child_alloc.height = alloc_length;
-
-              n_expand_children--;
-              expand_length_req -= length_req;
-              expand_length_avail -= alloc_length;
+              /* reset to new row if all columns are filled */
+              if (++col_count >= itembar->nrows)
+                {
+                  col_count = 0;
+                  row_max_size = 0;
+                }
             }
           else
             {
-              if (itembar->horizontal)
-                child_alloc.width = child_req.width;
+              /* reset column packing counters */
+              col_count = 0;
+              row_max_size = 0;
+
+              if (G_UNLIKELY (child->option == CHILD_OPTION_EXPAND))
+                {
+                  expand_len_req += child_len;
+                }
               else
-                child_alloc.height = child_req.height;
+                {
+                  expand_len_avail -= child_len;
+
+                  if (child->option == CHILD_OPTION_SHRINK)
+                    shrink_len_avail += child_len;
+                }
+            }
+        }
+      else
+        {
+          /* dnd separator */
+          expand_len_avail -= HIGHLIGHT_SIZE;
+        }
+    }
+
+  /* whether the expandable items fit on this row; we use this
+   * as a fast-path when there are expanding items on a panel with
+   * not really enough length to expand (ie. items make the panel grow,
+   * not the length set by the user) */
+  expand_children_fit = expand_len_req == expand_len_avail;
+
+  if (expand_len_avail < expand_len_req)
+    {
+      /* check if there are plugins on the panel we can shrink */
+      if (shrink_len_avail > 0)
+        shrink_len_req = expand_len_req - expand_len_avail;
+
+      expand_len_avail = expand_len_req;
+    }
+
+  /* init coordinates for first child */
+  x = x_init = allocation->x + border_width;
+  y = y_init = allocation->y + border_width;
+
+  /* init counters for small child packing */
+  row_max_size = 0;
+  col_count = 0;
+
+  /* the size property stored in the itembar is that of a single row */
+  rows_size = itembar->size * itembar->nrows;
+
+  /* allocate the children on this row */
+  for (lp = itembar->children; lp != NULL; lp = lp->next)
+    {
+      child = lp->data;
+
+      /* the highlight item for which we keep some spare space */
+      if (G_UNLIKELY (child == NULL))
+        {
+
+          if (IS_HORIZONTAL (itembar))
+            {
+              itembar->highlight_x = x;
+              itembar->highlight_y = y_init;
+
+              x += HIGHLIGHT_SIZE;
+            }
+          else
+            {
+              itembar->highlight_x = x_init;
+              itembar->highlight_y = y;
+
+              y += HIGHLIGHT_SIZE;
             }
 
-          if (itembar->horizontal)
+          expand_len_avail -= HIGHLIGHT_SIZE;
+
+          continue;
+        }
+
+      if (!GTK_WIDGET_VISIBLE (child->widget))
+        continue;
+
+      gtk_widget_get_child_requisition (child->widget, &child_req);
+
+      child_len = CHILD_LENGTH (child_req, itembar);
+
+      if (G_UNLIKELY (!expand_children_fit && child->option == CHILD_OPTION_EXPAND))
+        {
+          /* equally share the length between the expanding plugins */
+          panel_assert (expand_len_req > 0);
+          new_len = expand_len_avail * child_len / expand_len_req;
+
+          expand_len_req -= child_len;
+          expand_len_avail -= new_len;
+
+          child_len = new_len;
+        }
+      else if (child->option == CHILD_OPTION_SHRINK
+               && shrink_len_req > 0)
+        {
+          /* equally shrink all shrinking plugins */
+          panel_assert (shrink_len_avail > 0);
+          new_len = shrink_len_req * child_len / shrink_len_avail;
+
+          shrink_len_req -= new_len;
+          shrink_len_avail -= child_len;
+
+          child_len -= new_len;
+        }
+
+      if (G_UNLIKELY (child_len < 1))
+        child_len = 1;
+
+      if (child->option == CHILD_OPTION_SMALL
+          && itembar->nrows > 1
+          && CHILD_FITS_IN_ROW (child_req, itembar))
+        {
+          if (row_max_size < child_len)
+            row_max_size = child_len;
+
+          child_alloc.x = x;
+          child_alloc.y = y;
+
+          if (IS_HORIZONTAL (itembar))
             {
               child_alloc.height = itembar->size;
-              x += child_alloc.width;
+              child_alloc.width = child_len;
+
+              /* pack next small item below this one */
+              y += itembar->size;
             }
           else
             {
               child_alloc.width = itembar->size;
-              y += child_alloc.height;
+              child_alloc.height = child_len;
+
+              /* pack next time right of this one */
+              x += itembar->size;
             }
 
-          gtk_widget_size_allocate (child->widget, &child_alloc);
+          /* reset to new row if all columns are filled */
+          if (++col_count >= itembar->nrows)
+            {
+#define RESET_COLUMN_COUNTERS \
+              /* update coordinates */ \
+              if (IS_HORIZONTAL (itembar)) \
+                { \
+                  x += row_max_size; \
+                  y = y_init; \
+                } \
+              else \
+                { \
+                  y += row_max_size; \
+                  x = x_init; \
+                } \
+               \
+              col_count = 0; \
+              row_max_size = 0;
+
+              RESET_COLUMN_COUNTERS
+            }
         }
+      else
+        {
+          /* reset column packing counters */
+          if (col_count > 0)
+            {
+              RESET_COLUMN_COUNTERS
+            }
+
+          child_alloc.x = x;
+          child_alloc.y = y;
+
+          if (IS_HORIZONTAL (itembar))
+            {
+              child_alloc.height = rows_size;
+              child_alloc.width = child_len;
+
+              x += child_len;
+            }
+          else
+            {
+              child_alloc.width = rows_size;
+              child_alloc.height = child_len;
+
+              y += child_len;
+            }
+        }
+
+      gtk_widget_size_allocate (child->widget, &child_alloc);
     }
 }
 
@@ -533,20 +671,45 @@ static gboolean
 panel_itembar_expose_event (GtkWidget      *widget,
                             GdkEventExpose *event)
 {
-  gboolean      result;
   PanelItembar *itembar = PANEL_ITEMBAR (widget);
+  gboolean      result;
+  cairo_t      *cr;
+  GdkWindow    *window;
+  GdkRectangle  rect;
+  gint          row_size;
 
   result = (*GTK_WIDGET_CLASS (panel_itembar_parent_class)->expose_event) (widget, event);
 
   if (itembar->highlight_index != -1)
     {
-      gtk_paint_box (widget->style, widget->window,
-                     GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-                     &event->area, widget, "panel-dnd",
-                     itembar->highlight_x + 1,
-                     itembar->highlight_y + 1,
-                     itembar->size - 2,
-                     itembar->size - 2);
+      row_size = itembar->size * itembar->nrows;
+
+      rect.x = itembar->highlight_x;
+      rect.y = itembar->highlight_y;
+
+      if (IS_HORIZONTAL (itembar))
+        {
+          rect.width = HIGHLIGHT_SIZE;
+          rect.height = row_size;
+        }
+      else
+        {
+          rect.height = HIGHLIGHT_SIZE;
+          rect.width = row_size;
+        }
+
+      /* draw highlight box */
+      window = gtk_widget_get_window (widget);
+      cr = gdk_cairo_create (window);
+      cairo_set_source_rgb (cr, 1.00, 0.00, 0.00);
+
+      gdk_cairo_rectangle (cr, &event->area);
+      cairo_clip (cr);
+
+      gdk_cairo_rectangle (cr, &rect);
+      cairo_fill (cr);
+
+      cairo_destroy (cr);
     }
 
   return result;
@@ -632,7 +795,8 @@ panel_itembar_set_child_property (GtkContainer *container,
                                   GParamSpec   *pspec)
 {
   PanelItembarChild *child;
-  gboolean           boolean;
+  gboolean           enable;
+  ChildOptions       option;
 
   child = panel_itembar_get_child (PANEL_ITEMBAR (container), widget);
   if (G_UNLIKELY (child == NULL))
@@ -641,24 +805,35 @@ panel_itembar_set_child_property (GtkContainer *container,
   switch (prop_id)
     {
     case CHILD_PROP_EXPAND:
-      boolean = g_value_get_boolean (value);
-      if (child->expand == boolean)
-        return;
-      child->expand = boolean;
+      option = CHILD_OPTION_EXPAND;
       break;
 
-    case CHILD_PROP_WRAP:
-      boolean = g_value_get_boolean (value);
-      if (child->wrap == boolean)
-        return;
-      child->wrap = boolean;
+    case CHILD_PROP_SHRINK:
+      option = CHILD_OPTION_SHRINK;
+      break;
+
+    case CHILD_PROP_SMALL:
+      option = CHILD_OPTION_SMALL;
       break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (container, prop_id, pspec);
-      break;
+      return;
     }
 
+  /* warn if we override an old option */
+  if (child->option != CHILD_OPTION_NONE
+      && child->option != option)
+    g_warning ("Itembar child can only enable only of expand, shrink or small.");
+
+  /* leave if nothing changes */
+  enable = g_value_get_boolean (value);
+  if ((!enable && child->option == CHILD_OPTION_NONE)
+      || (enable && child->option == option))
+    return;
+
+  child->option = enable ? option : CHILD_OPTION_NONE;
+
   gtk_widget_queue_resize (GTK_WIDGET (container));
 }
 
@@ -680,11 +855,15 @@ panel_itembar_get_child_property (GtkContainer *container,
   switch (prop_id)
     {
     case CHILD_PROP_EXPAND:
-      g_value_set_boolean (value, child->expand);
+      g_value_set_boolean (value, child->option == CHILD_OPTION_EXPAND);
       break;
 
-    case CHILD_PROP_WRAP:
-      g_value_set_boolean (value, child->wrap);
+    case CHILD_PROP_SHRINK:
+      g_value_set_boolean (value, child->option == CHILD_OPTION_SHRINK);
+      break;
+
+    case CHILD_PROP_SMALL:
+      g_value_set_boolean (value, child->option == CHILD_OPTION_SMALL);
       break;
 
     default:
@@ -740,8 +919,7 @@ panel_itembar_insert (PanelItembar *itembar,
 
   child = g_slice_new0 (PanelItembarChild);
   child->widget = widget;
-  child->expand = FALSE;
-  child->wrap = FALSE;
+  child->option = CHILD_OPTION_NONE;
 
   itembar->children = g_slist_insert (itembar->children, child, position);
   gtk_widget_set_parent (widget, GTK_WIDGET (itembar));
@@ -829,7 +1007,6 @@ panel_itembar_get_drop_index (PanelItembar *itembar,
   GSList            *li;
   GtkAllocation     *alloc;
   guint              idx;
-  gint               row = 0;
 
   panel_return_val_if_fail (PANEL_IS_ITEMBAR (itembar), 0);
 
@@ -846,19 +1023,9 @@ panel_itembar_get_drop_index (PanelItembar *itembar,
       if (G_UNLIKELY (child == NULL))
         continue;
 
-      if (child->wrap)
-        {
-          row += itembar->size;
-
-          /* always make sure the item is on the row */
-          if ((itembar->horizontal && y < row)
-              || (!itembar->horizontal && x < row))
-            break;
-        }
-
       alloc = &child->widget->allocation;
 
-      if (itembar->horizontal)
+      if (IS_HORIZONTAL (itembar))
         {
           if (x < (alloc->x + (alloc->width / 2))
               && y >= alloc->y
