@@ -53,6 +53,7 @@
 #define SET_OLD_WM_STRUTS     (FALSE)
 #define DEFAULT_POPUP_DELAY   (225)
 #define DEFAULT_POPDOWN_DELAY (350)
+#define DEFAULT_ATUOHIDE_SIZE (3)
 #define HANDLE_SPACING        (4)
 #define HANDLE_DOTS           (2)
 #define HANDLE_PIXELS         (2)
@@ -60,6 +61,7 @@
 #define HANDLE_SIZE           (HANDLE_DOTS * (HANDLE_PIXELS + \
                                HANDLE_PIXEL_SPACE) - HANDLE_PIXEL_SPACE)
 #define HANDLE_SIZE_TOTAL     (2 * HANDLE_SPACING + HANDLE_SIZE)
+#define IS_HORIZONTAL(window) ((window)->mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL)
 
 
 
@@ -130,19 +132,24 @@ static void         panel_window_menu_popup                 (PanelWindow      *w
                                                              guint32           event_time);
 static void         panel_window_plugins_update             (PanelWindow      *window,
                                                              PluginProp        prop);
-static void         panel_window_plugin_set_orientation     (GtkWidget        *widget,
+static void         panel_window_plugin_set_mode            (GtkWidget        *widget,
                                                              gpointer          user_data);
 static void         panel_window_plugin_set_size            (GtkWidget        *widget,
                                                              gpointer          user_data);
+static void         panel_window_plugin_set_nrows           (GtkWidget        *widget,
+                                                             gpointer          user_data);
 static void         panel_window_plugin_set_screen_position (GtkWidget        *widget,
                                                              gpointer          user_data);
 
 
+
 enum
 {
   PROP_0,
-  PROP_HORIZONTAL,
+  PROP_ID,
+  PROP_MODE,
   PROP_SIZE,
+  PROP_NROWS,
   PROP_LENGTH,
   PROP_LENGTH_ADJUST,
   PROP_POSITION_LOCKED,
@@ -155,8 +162,9 @@ enum
 
 enum _PluginProp
 {
-  PLUGIN_PROP_ORIENTATION,
+  PLUGIN_PROP_MODE,
   PLUGIN_PROP_SCREEN_POSITION,
+  PLUGIN_PROP_NROWS,
   PLUGIN_PROP_SIZE
 };
 
@@ -238,6 +246,9 @@ struct _PanelWindow
 {
   PanelBaseWindow      __parent__;
 
+  /* unique id of this panel */
+  gint                 id;
+
   /* whether the user is allowed to make
    * changes to this window */
   guint                locked : 1;
@@ -255,7 +266,8 @@ struct _PanelWindow
   guint                size;
   gdouble              length;
   guint                length_adjust : 1;
-  guint                horizontal : 1;
+  XfcePanelPluginMode  mode;
+  guint                nrows;
   SnapPosition         snap_position;
   guint                span_monitors : 1;
   gchar               *output_name;
@@ -269,6 +281,7 @@ struct _PanelWindow
   guint                autohide_timeout_id;
   gint                 autohide_block;
   gint                 autohide_grab_block;
+  gint                 autohide_size;
 
   /* popup/down delay from gtk style */
   gint                 popup_delay;
@@ -333,10 +346,18 @@ panel_window_class_init (PanelWindowClass *klass)
   gtkwidget_class->realize = panel_window_realize;
 
   g_object_class_install_property (gobject_class,
-                                   PROP_HORIZONTAL,
-                                   g_param_spec_boolean ("horizontal", NULL, NULL,
-                                                         TRUE,
-                                                         EXO_PARAM_READWRITE));
+                                   PROP_ID,
+                                   g_param_spec_int ("id", NULL, NULL,
+                                                     0, G_MAXINT, 0,
+                                                     EXO_PARAM_READWRITE
+                                                     | G_PARAM_CONSTRUCT_ONLY));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_MODE,
+                                   g_param_spec_enum ("mode", NULL, NULL,
+                                                      XFCE_TYPE_PANEL_PLUGIN_MODE,
+                                                      XFCE_PANEL_PLUGIN_MODE_HORIZONTAL,
+                                                      EXO_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_SIZE,
@@ -344,10 +365,16 @@ panel_window_class_init (PanelWindowClass *klass)
                                                       16, 128, 48,
                                                       EXO_PARAM_READWRITE));
 
+  g_object_class_install_property (gobject_class,
+                                   PROP_NROWS,
+                                   g_param_spec_uint ("nrows", NULL, NULL,
+                                                      1, 6, 1,
+                                                      EXO_PARAM_READWRITE));
+
   g_object_class_install_property (gobject_class,
                                    PROP_LENGTH,
                                    g_param_spec_uint ("length", NULL, NULL,
-                                                      1, 100, 25,
+                                                      1, 100, 10,
                                                       EXO_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
@@ -408,6 +435,14 @@ panel_window_class_init (PanelWindowClass *klass)
                                                              DEFAULT_POPDOWN_DELAY,
                                                              EXO_PARAM_READABLE));
 
+  gtk_widget_class_install_style_property (gtkwidget_class,
+                                           g_param_spec_int ("autohide-size",
+                                                             NULL,
+                                                             "Size of hidden panel",
+                                                             1, G_MAXINT,
+                                                             DEFAULT_ATUOHIDE_SIZE,
+                                                             EXO_PARAM_READABLE));
+
   /* initialize the atoms */
   cardinal_atom = gdk_atom_intern_static_string ("CARDINAL");
   net_wm_strut_partial_atom = gdk_atom_intern_static_string ("_NET_WM_STRUT_PARTIAL");
@@ -421,12 +456,14 @@ panel_window_class_init (PanelWindowClass *klass)
 static void
 panel_window_init (PanelWindow *window)
 {
+  window->id = -1;
   window->locked = TRUE;
   window->screen = NULL;
   window->struts_edge = STRUTS_EDGE_NONE;
   window->struts_disabled = FALSE;
-  window->horizontal = TRUE;
-  window->size = 30;
+  window->mode = XFCE_PANEL_PLUGIN_MODE_HORIZONTAL;
+  window->size = 48;
+  window->nrows = 1;
   window->length = 0.10;
   window->length_adjust = TRUE;
   window->snap_position = SNAP_POSITION_NONE;
@@ -436,6 +473,7 @@ panel_window_init (PanelWindow *window)
   window->autohide_timeout_id = 0;
   window->autohide_block = 0;
   window->autohide_grab_block = 0;
+  window->autohide_size = DEFAULT_ATUOHIDE_SIZE;
   window->popup_delay = DEFAULT_POPUP_DELAY;
   window->popdown_delay = DEFAULT_POPDOWN_DELAY;
   window->base_x = -1;
@@ -470,14 +508,22 @@ panel_window_get_property (GObject    *object,
 
   switch (prop_id)
     {
-    case PROP_HORIZONTAL:
-      g_value_set_boolean (value, window->horizontal);
+    case PROP_ID:
+      g_value_set_int (value, window->id);
+      break;
+
+    case PROP_MODE:
+      g_value_set_enum (value, window->mode);
       break;
 
     case PROP_SIZE:
       g_value_set_uint (value, window->size);
       break;
 
+    case PROP_NROWS:
+      g_value_set_uint (value, window->nrows);
+      break;
+
     case PROP_LENGTH:
       g_value_set_uint (value,  rint (window->length * 100.00));
       break;
@@ -528,26 +574,31 @@ panel_window_set_property (GObject      *object,
                            const GValue *value,
                            GParamSpec   *pspec)
 {
-  PanelWindow *window = PANEL_WINDOW (object);
-  gboolean     val_bool;
-  guint        val_uint;
-  gdouble      val_double;
-  const gchar *val_string;
-  gboolean     update = FALSE;
-  gint         x, y, snap_position;
+  PanelWindow         *window = PANEL_WINDOW (object);
+  gboolean             val_bool;
+  guint                val_uint;
+  gdouble              val_double;
+  const gchar         *val_string;
+  gboolean             update = FALSE;
+  gint                 x, y, snap_position;
+  XfcePanelPluginMode  val_mode;
 
   switch (prop_id)
     {
-    case PROP_HORIZONTAL:
-      val_bool = g_value_get_boolean (value);
-      if (window->horizontal != val_bool)
+    case PROP_ID:
+      window->id = g_value_get_int (value);
+      break;
+
+    case PROP_MODE:
+      val_mode = g_value_get_enum (value);
+      if (window->mode != val_mode)
         {
-          window->horizontal = !!val_bool;
+          window->mode = val_mode;
           panel_window_screen_layout_changed (window->screen, window);
         }
 
       /* send the new orientation and screen position to the panel plugins */
-      panel_window_plugins_update (window, PLUGIN_PROP_ORIENTATION);
+      panel_window_plugins_update (window, PLUGIN_PROP_MODE);
       panel_window_plugins_update (window, PLUGIN_PROP_SCREEN_POSITION);
       break;
 
@@ -563,6 +614,18 @@ panel_window_set_property (GObject      *object,
       panel_window_plugins_update (window, PLUGIN_PROP_SIZE);
       break;
 
+    case PROP_NROWS:
+      val_uint = g_value_get_uint (value);
+      if (window->nrows != val_uint)
+        {
+          window->nrows = val_uint;
+          gtk_widget_queue_resize (GTK_WIDGET (window));
+        }
+
+      /* send the new size to the panel plugins */
+      panel_window_plugins_update (window, PLUGIN_PROP_NROWS);
+      break;
+
     case PROP_LENGTH:
       val_double = g_value_get_uint (value) / 100.00;
       if (window->length != val_double)
@@ -628,8 +691,8 @@ panel_window_set_property (GObject      *object,
           && sscanf (val_string, "p=%d;x=%d;y=%d", &snap_position, &x, &y) == 3)
         {
           window->snap_position = CLAMP (snap_position, SNAP_POSITION_NONE, SNAP_POSITION_S);
-          window->base_x = x;
-          window->base_y = y;
+          window->base_x = MAX (x, 0);
+          window->base_y = MAX (y, 0);
 
           panel_window_screen_layout_changed (window->screen, window);
 
@@ -638,7 +701,7 @@ panel_window_set_property (GObject      *object,
         }
       else
         {
-          g_message ("no valid position defined: %s", val_string);
+          g_message ("Not a valid position defined: %s", val_string);
         }
       break;
 
@@ -698,7 +761,7 @@ panel_window_expose_event (GtkWidget      *widget,
   if (window->position_locked || !GTK_WIDGET_DRAWABLE (widget))
     goto end;
 
-  if (window->horizontal)
+  if (IS_HORIZONTAL (window))
     {
       handle_h = window->alloc.height / 2;
       handle_w = HANDLE_SIZE;
@@ -1091,7 +1154,7 @@ panel_window_size_request (GtkWidget      *widget,
   /* handle size */
   if (!window->position_locked)
     {
-      if (window->horizontal)
+      if (IS_HORIZONTAL (window))
         extra_width += 2 * HANDLE_SIZE_TOTAL;
       else
         extra_height += 2 * HANDLE_SIZE_TOTAL;
@@ -1112,7 +1175,7 @@ panel_window_size_request (GtkWidget      *widget,
   requisition->width = child_requisition.width + extra_width;
 
   /* respect the length and monitor/screen size */
-  if (window->horizontal)
+  if (IS_HORIZONTAL (window))
     {
       if (!window->length_adjust)
         requisition->width = extra_width;
@@ -1152,7 +1215,7 @@ panel_window_size_allocate (GtkWidget     *widget,
       window->alloc.x = window->alloc.y = -9999;
 
       /* set hidden window size */
-      w = h = 3;
+      w = h = window->autohide_size;
 
       switch (window->snap_position)
         {
@@ -1174,7 +1237,7 @@ panel_window_size_allocate (GtkWidget     *widget,
 
         /* corner or floating panel */
         default:
-          if (window->horizontal)
+          if (IS_HORIZONTAL (window))
             w = alloc->width;
           else
             h = alloc->height;
@@ -1233,7 +1296,7 @@ panel_window_size_allocate (GtkWidget     *widget,
       /* keep space for the panel handles if not locked */
       if (!window->position_locked)
         {
-          if (window->horizontal)
+          if (IS_HORIZONTAL (window))
             {
               child_alloc.width -= 2 * HANDLE_SIZE_TOTAL;
               child_alloc.x += HANDLE_SIZE_TOTAL;
@@ -1388,6 +1451,7 @@ panel_window_style_set (GtkWidget *widget,
   gtk_widget_style_get (GTK_WIDGET (widget),
                         "popup-delay", &window->popup_delay,
                         "popdown-delay", &window->popdown_delay,
+                        "autohide-size", &window->autohide_size,
                         NULL);
 }
 
@@ -1426,31 +1490,31 @@ panel_window_screen_struts_edge (PanelWindow *window)
 
     case SNAP_POSITION_E:
     case SNAP_POSITION_EC:
-      return window->horizontal ? STRUTS_EDGE_NONE : STRUTS_EDGE_RIGHT;
+      return IS_HORIZONTAL (window) ? STRUTS_EDGE_NONE : STRUTS_EDGE_RIGHT;
 
     case SNAP_POSITION_NE:
-      return window->horizontal ? STRUTS_EDGE_TOP : STRUTS_EDGE_RIGHT;
+      return IS_HORIZONTAL (window) ? STRUTS_EDGE_TOP : STRUTS_EDGE_RIGHT;
 
     case SNAP_POSITION_SE:
-      return window->horizontal ? STRUTS_EDGE_BOTTOM : STRUTS_EDGE_RIGHT;
+      return IS_HORIZONTAL (window) ? STRUTS_EDGE_BOTTOM : STRUTS_EDGE_RIGHT;
 
     case SNAP_POSITION_W:
     case SNAP_POSITION_WC:
-      return window->horizontal ? STRUTS_EDGE_NONE : STRUTS_EDGE_LEFT;
+      return IS_HORIZONTAL (window) ? STRUTS_EDGE_NONE : STRUTS_EDGE_LEFT;
 
     case SNAP_POSITION_NW:
-      return window->horizontal ? STRUTS_EDGE_TOP : STRUTS_EDGE_LEFT;
+      return IS_HORIZONTAL (window) ? STRUTS_EDGE_TOP : STRUTS_EDGE_LEFT;
 
     case SNAP_POSITION_SW:
-      return window->horizontal ? STRUTS_EDGE_BOTTOM : STRUTS_EDGE_LEFT;
+      return IS_HORIZONTAL (window) ? STRUTS_EDGE_BOTTOM : STRUTS_EDGE_LEFT;
 
     case SNAP_POSITION_NC:
     case SNAP_POSITION_N:
-      return window->horizontal ? STRUTS_EDGE_TOP : STRUTS_EDGE_NONE;
+      return IS_HORIZONTAL (window) ? STRUTS_EDGE_TOP : STRUTS_EDGE_NONE;
 
     case SNAP_POSITION_SC:
     case SNAP_POSITION_S:
-      return window->horizontal ? STRUTS_EDGE_BOTTOM : STRUTS_EDGE_NONE;
+      return IS_HORIZONTAL (window) ? STRUTS_EDGE_BOTTOM : STRUTS_EDGE_NONE;
     }
 
   return STRUTS_EDGE_NONE;
@@ -1632,7 +1696,7 @@ panel_window_screen_update_borders (PanelWindow *window)
   /* don't show the side borders if the length is 100% */
   if (window->length == 1.00)
     {
-      if (window->horizontal)
+      if (IS_HORIZONTAL (window))
         PANEL_UNSET_FLAG (borders, PANEL_BORDER_LEFT | PANEL_BORDER_RIGHT);
       else
         PANEL_UNSET_FLAG (borders, PANEL_BORDER_TOP | PANEL_BORDER_BOTTOM);
@@ -2344,18 +2408,21 @@ panel_window_plugins_update (PanelWindow *window,
 
   switch (prop)
     {
-    case PLUGIN_PROP_ORIENTATION:
-      func = panel_window_plugin_set_orientation;
+    case PLUGIN_PROP_MODE:
+      func = panel_window_plugin_set_mode;
       break;
 
     case PLUGIN_PROP_SCREEN_POSITION:
       func = panel_window_plugin_set_screen_position;
       break;
 
+    case PLUGIN_PROP_NROWS:
+      func = panel_window_plugin_set_nrows;
+      break;
+
     case PLUGIN_PROP_SIZE:
       func = panel_window_plugin_set_size;
       break;
-      break;
 
     default:
       panel_assert_not_reached ();
@@ -2373,15 +2440,14 @@ panel_window_plugins_update (PanelWindow *window,
 
 
 static void
-panel_window_plugin_set_orientation (GtkWidget *widget,
-                                     gpointer   user_data)
+panel_window_plugin_set_mode (GtkWidget *widget,
+                              gpointer   user_data)
 {
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (widget));
   panel_return_if_fail (PANEL_IS_WINDOW (user_data));
 
-  xfce_panel_plugin_provider_set_orientation (XFCE_PANEL_PLUGIN_PROVIDER (widget),
-      PANEL_WINDOW (user_data)->horizontal ? GTK_ORIENTATION_HORIZONTAL:
-          GTK_ORIENTATION_VERTICAL);
+  xfce_panel_plugin_provider_set_mode (XFCE_PANEL_PLUGIN_PROVIDER (widget),
+                                       PANEL_WINDOW (user_data)->mode);
 }
 
 
@@ -2399,6 +2465,19 @@ panel_window_plugin_set_size (GtkWidget *widget,
 
 
 
+static void
+panel_window_plugin_set_nrows (GtkWidget *widget,
+                               gpointer   user_data)
+{
+  panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (widget));
+  panel_return_if_fail (PANEL_IS_WINDOW (user_data));
+
+  xfce_panel_plugin_provider_set_nrows (XFCE_PANEL_PLUGIN_PROVIDER (widget),
+                                        PANEL_WINDOW (user_data)->nrows);
+}
+
+
+
 static void
 panel_window_plugin_set_screen_position (GtkWidget *widget,
                                          gpointer   user_data)
@@ -2412,51 +2491,51 @@ panel_window_plugin_set_screen_position (GtkWidget *widget,
   switch (window->snap_position)
     {
     case SNAP_POSITION_NONE:
-      position = window->horizontal ? XFCE_SCREEN_POSITION_FLOATING_H :
+      position = IS_HORIZONTAL (window) ? XFCE_SCREEN_POSITION_FLOATING_H :
           XFCE_SCREEN_POSITION_FLOATING_V;
       break;
 
     case SNAP_POSITION_NW:
-      position = window->horizontal ? XFCE_SCREEN_POSITION_NW_H :
+      position = IS_HORIZONTAL (window) ? XFCE_SCREEN_POSITION_NW_H :
           XFCE_SCREEN_POSITION_NW_V;
       break;
 
     case SNAP_POSITION_NE:
-      position = window->horizontal ? XFCE_SCREEN_POSITION_NE_H :
+      position = IS_HORIZONTAL (window) ? XFCE_SCREEN_POSITION_NE_H :
           XFCE_SCREEN_POSITION_NE_V;
       break;
 
     case SNAP_POSITION_SW:
-      position = window->horizontal ? XFCE_SCREEN_POSITION_SW_H :
+      position = IS_HORIZONTAL (window) ? XFCE_SCREEN_POSITION_SW_H :
           XFCE_SCREEN_POSITION_SW_V;
       break;
 
     case SNAP_POSITION_SE:
-      position = window->horizontal ? XFCE_SCREEN_POSITION_SE_H :
+      position = IS_HORIZONTAL (window) ? XFCE_SCREEN_POSITION_SE_H :
           XFCE_SCREEN_POSITION_SE_V;
       break;
 
     case SNAP_POSITION_W:
     case SNAP_POSITION_WC:
-      position = window->horizontal ? XFCE_SCREEN_POSITION_FLOATING_H :
+      position = IS_HORIZONTAL (window) ? XFCE_SCREEN_POSITION_FLOATING_H :
           XFCE_SCREEN_POSITION_W;
       break;
 
     case SNAP_POSITION_E:
     case SNAP_POSITION_EC:
-      position = window->horizontal ? XFCE_SCREEN_POSITION_FLOATING_H :
+      position = IS_HORIZONTAL (window) ? XFCE_SCREEN_POSITION_FLOATING_H :
           XFCE_SCREEN_POSITION_E;
       break;
 
     case SNAP_POSITION_S:
     case SNAP_POSITION_SC:
-      position = window->horizontal ? XFCE_SCREEN_POSITION_S :
+      position = IS_HORIZONTAL (window) ? XFCE_SCREEN_POSITION_S :
           XFCE_SCREEN_POSITION_FLOATING_V;
       break;
 
     case SNAP_POSITION_N:
     case SNAP_POSITION_NC:
-      position = window->horizontal ? XFCE_SCREEN_POSITION_N :
+      position = IS_HORIZONTAL (window) ? XFCE_SCREEN_POSITION_N :
           XFCE_SCREEN_POSITION_FLOATING_V;
       break;
 
@@ -2472,12 +2551,14 @@ panel_window_plugin_set_screen_position (GtkWidget *widget,
 
 
 GtkWidget *
-panel_window_new (GdkScreen *screen)
+panel_window_new (GdkScreen *screen,
+                  gint       id)
 {
   if (screen == NULL)
     screen = gdk_screen_get_default ();
 
   return g_object_new (PANEL_TYPE_WINDOW,
+                       "id", id,
                        "type", GTK_WINDOW_TOPLEVEL,
                        "decorated", FALSE,
                        "resizable", FALSE,
@@ -2491,6 +2572,25 @@ panel_window_new (GdkScreen *screen)
 
 
 
+gint
+panel_window_get_id (PanelWindow *window)
+{
+  panel_return_val_if_fail (PANEL_IS_WINDOW (window), -1);
+  panel_return_val_if_fail (window->id > -1, -1);
+  return window->id;
+}
+
+
+
+gboolean
+panel_window_has_position (PanelWindow *window)
+{
+  panel_return_val_if_fail (PANEL_IS_WINDOW (window), FALSE);
+  return window->base_x != -1 && window->base_y != -1;
+}
+
+
+
 void
 panel_window_set_povider_info (PanelWindow *window,
                                GtkWidget   *provider,
@@ -2529,9 +2629,10 @@ panel_window_set_povider_info (PanelWindow *window,
         }
     }
 
-  panel_window_plugin_set_orientation (provider, window);
+  panel_window_plugin_set_mode (provider, window);
   panel_window_plugin_set_screen_position (provider, window);
   panel_window_plugin_set_size (provider, window);
+  panel_window_plugin_set_nrows (provider, window);
 }
 
 
