@@ -101,14 +101,15 @@ static void      applications_menu_plugin_construct            (XfcePanelPlugin
 static void      applications_menu_plugin_free_data            (XfcePanelPlugin        *panel_plugin);
 static gboolean  applications_menu_plugin_size_changed         (XfcePanelPlugin        *panel_plugin,
                                                                 gint                    size);
-static void      applications_menu_plugin_orientation_changed  (XfcePanelPlugin        *panel_plugin,
-                                                                GtkOrientation          orientation);
+static void      applications_menu_plugin_mode_changed         (XfcePanelPlugin        *panel_plugin,
+                                                                XfcePanelPluginMode     mode);
 static void      applications_menu_plugin_configure_plugin     (XfcePanelPlugin        *panel_plugin);
 static gboolean  applications_menu_plugin_remote_event         (XfcePanelPlugin        *panel_plugin,
                                                                 const gchar            *name,
                                                                 const GValue           *value);
 static void      applications_menu_plugin_menu_reload          (ApplicationsMenuPlugin *plugin);
-static void      applications_menu_plugin_menu                 (GtkWidget              *button,
+static gboolean  applications_menu_plugin_menu                 (GtkWidget              *button,
+                                                                GdkEventButton         *event,
                                                                 ApplicationsMenuPlugin *plugin);
 
 
@@ -136,7 +137,7 @@ applications_menu_plugin_class_init (ApplicationsMenuPluginClass *klass)
   plugin_class->construct = applications_menu_plugin_construct;
   plugin_class->free_data = applications_menu_plugin_free_data;
   plugin_class->size_changed = applications_menu_plugin_size_changed;
-  plugin_class->orientation_changed = applications_menu_plugin_orientation_changed;
+  plugin_class->mode_changed = applications_menu_plugin_mode_changed;
   plugin_class->configure_plugin = applications_menu_plugin_configure_plugin;
   plugin_class->remote_event = applications_menu_plugin_remote_event;
 
@@ -207,15 +208,25 @@ applications_menu_plugin_class_init (ApplicationsMenuPluginClass *klass)
 static void
 applications_menu_plugin_init (ApplicationsMenuPlugin *plugin)
 {
+  const gchar *desktop;
+
   plugin->show_menu_icons = TRUE;
   plugin->show_button_title = TRUE;
   plugin->custom_menu = FALSE;
 
-  panel_debug (PANEL_DEBUG_APPLICATIONSMENU,
-               "XDG_MENU_PREFIX is set to \"%s\"",
-               g_getenv ("XDG_MENU_PREFIX"));
+  /* if the value is unset, fallback to XFCE, if the
+   * value is empty, allow all applications in the menu */
+  desktop = g_getenv ("XDG_CURRENT_DESKTOP");
+  if (G_LIKELY (desktop == NULL))
+    desktop = "XFCE";
+  else if (*desktop == '\0')
+    desktop = NULL;
 
-  garcon_set_environment ("XFCE");
+  panel_debug (PANEL_DEBUG_APPLICATIONSMENU,
+               "XDG_MENU_PREFIX is set to \"%s\", menu environment is \"%s\"",
+               g_getenv ("XDG_MENU_PREFIX"), desktop);
+
+  garcon_set_environment (desktop);
 
   plugin->button = xfce_panel_create_toggle_button ();
   xfce_panel_plugin_add_action_widget (XFCE_PANEL_PLUGIN (plugin), plugin->button);
@@ -223,7 +234,7 @@ applications_menu_plugin_init (ApplicationsMenuPlugin *plugin)
   gtk_widget_set_name (plugin->button, "applicationmenu-button");
   gtk_button_set_relief (GTK_BUTTON (plugin->button), GTK_RELIEF_NONE);
   gtk_widget_set_tooltip_text (plugin->button, DEFAULT_TITLE);
-  g_signal_connect (G_OBJECT (plugin->button), "toggled",
+  g_signal_connect (G_OBJECT (plugin->button), "button-press-event",
       G_CALLBACK (applications_menu_plugin_menu), plugin);
 
   plugin->box = xfce_hvbox_new (GTK_ORIENTATION_HORIZONTAL, FALSE, 1);
@@ -332,6 +343,14 @@ applications_menu_plugin_set_property (GObject      *object,
           plugin->button_title != NULL ? plugin->button_title : "");
       gtk_widget_set_tooltip_text (plugin->button,
           exo_str_is_empty (plugin->button_title) ? NULL : plugin->button_title);
+
+      /* check if the label still fits */
+      if (xfce_panel_plugin_get_mode (XFCE_PANEL_PLUGIN (plugin)) == XFCE_PANEL_PLUGIN_MODE_DESKBAR
+          && plugin->show_button_title)
+        {
+          applications_menu_plugin_size_changed (XFCE_PANEL_PLUGIN (plugin),
+              xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin)));
+        }
       return;
 
     case PROP_BUTTON_ICON:
@@ -409,39 +428,73 @@ applications_menu_plugin_size_changed (XfcePanelPlugin *panel_plugin,
                                        gint             size)
 {
   ApplicationsMenuPlugin *plugin = XFCE_APPLICATIONS_MENU_PLUGIN (panel_plugin);
+  gint                    row_size;
   gint                    icon_size;
   GtkStyle               *style;
+  XfcePanelPluginMode     mode;
+  GtkRequisition          label_size;
+  GtkOrientation          orientation;
+  gint                    border_thickness;
+
+  row_size = size / xfce_panel_plugin_get_nrows (panel_plugin);
 
   gtk_box_set_child_packing (GTK_BOX (plugin->box), plugin->icon,
                              !plugin->show_button_title, !plugin->show_button_title,
                              0, GTK_PACK_START);
 
+  mode = xfce_panel_plugin_get_mode (panel_plugin);
+
+  if (mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL)
+    orientation = GTK_ORIENTATION_HORIZONTAL;
+  else
+    orientation = GTK_ORIENTATION_VERTICAL;
+
   if (!plugin->show_button_title)
     {
       xfce_panel_image_set_size (XFCE_PANEL_IMAGE (plugin->icon), -1);
-      gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), size, size);
+
+      if (mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL)
+        gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), row_size, size);
+      else
+        gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), size, row_size);
     }
   else
     {
       style = gtk_widget_get_style (plugin->button);
-      icon_size = size - 2 * MAX (style->xthickness, style->ythickness);
+      border_thickness = 2 * MAX (style->xthickness, style->ythickness) + 2;
+
+      icon_size = row_size - border_thickness;
       xfce_panel_image_set_size (XFCE_PANEL_IMAGE (plugin->icon), icon_size);
       gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), -1, -1);
+
+      if (mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR)
+        {
+          /* check if the label fits next to the icon */
+          gtk_widget_size_request (GTK_WIDGET (plugin->label), &label_size);
+          if (label_size.width <= size - border_thickness - icon_size)
+            orientation = GTK_ORIENTATION_HORIZONTAL;
+        }
     }
 
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (plugin->box), orientation);
+
   return TRUE;
 }
 
 
 
 static void
-applications_menu_plugin_orientation_changed (XfcePanelPlugin *panel_plugin,
-                                              GtkOrientation   orientation)
+applications_menu_plugin_mode_changed (XfcePanelPlugin     *panel_plugin,
+                                       XfcePanelPluginMode  mode)
 {
   ApplicationsMenuPlugin *plugin = XFCE_APPLICATIONS_MENU_PLUGIN (panel_plugin);
+  gint                    angle;
 
-  xfce_hvbox_set_orientation (XFCE_HVBOX (plugin->box), orientation);
-  gtk_label_set_angle (GTK_LABEL (plugin->label), orientation == GTK_ORIENTATION_HORIZONTAL ? 0 : 270);
+  angle = (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL) ? 270 : 0;
+  gtk_label_set_angle (GTK_LABEL (plugin->label), angle);
+
+  applications_menu_plugin_size_changed (panel_plugin,
+      xfce_panel_plugin_get_size (panel_plugin));
 }
 
 
@@ -508,7 +561,8 @@ applications_menu_plugin_configure_plugin_edit (GtkWidget              *button,
   panel_return_if_fail (XFCE_IS_APPLICATIONS_MENU_PLUGIN (plugin));
   panel_return_if_fail (GTK_IS_WIDGET (button));
 
-  if (!gdk_spawn_command_line_on_screen (gtk_widget_get_screen (button), command, &error))
+  if (!xfce_spawn_command_line_on_screen (gtk_widget_get_screen (button), command,
+                                          FALSE, FALSE, &error))
     {
       xfce_dialog_show_error (NULL, error, _("Failed to execute command \"%s\"."), command);
       g_error_free (error);
@@ -619,12 +673,12 @@ applications_menu_plugin_remote_event (XfcePanelPlugin *panel_plugin,
           && g_value_get_boolean (value))
         {
           /* show menu under cursor */
-          applications_menu_plugin_menu (NULL, plugin);
+          applications_menu_plugin_menu (NULL, NULL, plugin);
         }
       else
         {
           /* show the menu at the button */
-          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->button), TRUE);
+          applications_menu_plugin_menu (plugin->button, NULL, plugin);
         }
 
       /* don't popup another menu */
@@ -941,6 +995,9 @@ applications_menu_plugin_menu_add (GtkWidget              *gtk_menu,
               gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), submenu);
               gtk_widget_show (mi);
 
+              g_signal_connect_swapped (G_OBJECT (li->data), "directory-changed",
+                  G_CALLBACK (applications_menu_plugin_menu_reload), plugin);
+
               if (plugin->show_menu_icons)
                 {
                   icon_name = garcon_menu_element_get_icon_name (li->data);
@@ -969,8 +1026,9 @@ applications_menu_plugin_menu_add (GtkWidget              *gtk_menu,
 
 
 
-static void
+static gboolean
 applications_menu_plugin_menu (GtkWidget              *button,
+                               GdkEventButton         *event,
                                ApplicationsMenuPlugin *plugin)
 {
   GtkWidget  *mi;
@@ -979,12 +1037,17 @@ applications_menu_plugin_menu (GtkWidget              *button,
   gchar      *filename;
   GFile      *file;
 
-  panel_return_if_fail (XFCE_IS_APPLICATIONS_MENU_PLUGIN (plugin));
-  panel_return_if_fail (button == NULL || plugin->button == button);
+  panel_return_val_if_fail (XFCE_IS_APPLICATIONS_MENU_PLUGIN (plugin), FALSE);
+  panel_return_val_if_fail (button == NULL || plugin->button == button, FALSE);
 
-  if (button != NULL
-      && !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
-    return;
+  if (event != NULL
+      && !(event->button == 1
+           && event->type == GDK_BUTTON_PRESS
+           && !PANEL_HAS_FLAG (event->state, GDK_CONTROL_MASK)))
+    return FALSE;
+
+  if (button != NULL)
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 
   if (plugin->menu == NULL)
     {
@@ -1032,18 +1095,22 @@ applications_menu_plugin_menu (GtkWidget              *button,
         {
           xfce_dialog_show_error (NULL, error, _("Failed to load the applications menu"));
 
-          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
+          if (button != NULL)
+            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
 
           if (G_LIKELY (error != NULL))
             g_error_free (error);
           if (G_LIKELY (menu != NULL))
             g_object_unref (G_OBJECT (menu));
 
-          return;
+          return FALSE;
         }
     }
 
   gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
                   button != NULL ? xfce_panel_plugin_position_menu : NULL,
-                  plugin, 1, gtk_get_current_event_time ());
+                  plugin, 1,
+                  event != NULL ? event->time : gtk_get_current_event_time ());
+
+  return TRUE;
 }
