@@ -85,6 +85,8 @@ static void      directory_menu_plugin_set_property         (GObject
 static void      directory_menu_plugin_construct            (XfcePanelPlugin     *panel_plugin);
 static void      directory_menu_plugin_free_file_patterns   (DirectoryMenuPlugin *plugin);
 static void      directory_menu_plugin_free_data            (XfcePanelPlugin     *panel_plugin);
+static gboolean  directory_menu_plugin_size_changed         (XfcePanelPlugin     *panel_plugin,
+                                                             gint                 size);
 static void      directory_menu_plugin_configure_plugin     (XfcePanelPlugin     *panel_plugin);
 static gboolean  directory_menu_plugin_remote_event         (XfcePanelPlugin     *panel_plugin,
                                                              const gchar         *name,
@@ -116,6 +118,7 @@ directory_menu_plugin_class_init (DirectoryMenuPluginClass *klass)
   plugin_class = XFCE_PANEL_PLUGIN_CLASS (klass);
   plugin_class->construct = directory_menu_plugin_construct;
   plugin_class->free_data = directory_menu_plugin_free_data;
+  plugin_class->size_changed = directory_menu_plugin_size_changed;
   plugin_class->configure_plugin = directory_menu_plugin_configure_plugin;
   plugin_class->remote_event = directory_menu_plugin_remote_event;
 
@@ -295,7 +298,9 @@ directory_menu_plugin_construct (XfcePanelPlugin *panel_plugin)
     { NULL }
   };
 
-  xfce_panel_plugin_menu_show_configure (XFCE_PANEL_PLUGIN (plugin));
+  xfce_panel_plugin_menu_show_configure (panel_plugin);
+
+  xfce_panel_plugin_set_small (panel_plugin, TRUE);
 
   /* bind all properties */
   panel_properties_bind (NULL, G_OBJECT (plugin),
@@ -340,6 +345,20 @@ directory_menu_plugin_free_data (XfcePanelPlugin *panel_plugin)
 
 
 
+static gboolean
+directory_menu_plugin_size_changed (XfcePanelPlugin *panel_plugin,
+                                    gint             size)
+{
+  /* force a square button */
+  size /= xfce_panel_plugin_get_nrows (panel_plugin);
+  gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), size, size);
+
+  return TRUE;
+}
+
+
+
+
 static void
 directory_menu_plugin_configure_plugin_file_set (GtkFileChooserButton *button,
                                                  DirectoryMenuPlugin  *plugin)
@@ -620,7 +639,8 @@ err:
 static void
 directory_menu_plugin_menu_open (GtkWidget   *mi,
                                  GFile       *dir,
-                                 const gchar *category)
+                                 const gchar *category,
+                                 gboolean     path_as_arg)
 {
   GError       *error = NULL;
   gchar        *working_dir;
@@ -673,7 +693,7 @@ directory_menu_plugin_menu_open (GtkWidget   *mi,
             continue;
 
           argv[0] = filename;
-          argv[1] = working_dir;
+          argv[1] = path_as_arg ? working_dir : NULL;
           argv[2] = NULL;
 
           /* try to spawn the program, if this fails we try exo for
@@ -710,7 +730,7 @@ directory_menu_plugin_menu_open_terminal (GtkWidget *mi,
   panel_return_if_fail (GTK_IS_WIDGET (mi));
   panel_return_if_fail (G_IS_FILE (dir));
 
-  directory_menu_plugin_menu_open (mi, dir, "TerminalEmulator");
+  directory_menu_plugin_menu_open (mi, dir, "TerminalEmulator", FALSE);
 }
 
 
@@ -722,7 +742,7 @@ directory_menu_plugin_menu_open_folder (GtkWidget *mi,
   panel_return_if_fail (GTK_IS_WIDGET (mi));
   panel_return_if_fail (G_IS_FILE (dir));
 
-  directory_menu_plugin_menu_open (mi, dir, "FileManager");
+  directory_menu_plugin_menu_open (mi, dir, "FileManager", TRUE);
 }
 
 
