@@ -420,7 +420,8 @@ pager_plugin_configure_workspace_settings (GtkWidget *button)
     screen = gdk_screen_get_default ();
 
   /* try to start the settings dialog */
-  if (!gdk_spawn_command_line_on_screen (screen, WORKSPACE_SETTINGS_COMMAND, &error))
+  if (!xfce_spawn_command_line_on_screen (screen, WORKSPACE_SETTINGS_COMMAND,
+                                          FALSE, FALSE, &error))
     {
       /* show an error dialog */
       toplevel = gtk_widget_get_toplevel (button);
